/* Copyright 2021
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "config_common.h"

/*
 * https://learn.adafruit.com/adafruit-feather-32u4-bluefruit-le/pinouts
 * 
 * 
 * Verbindungen zwischen mega und ble, die auch auf den äußeren Anschlußleisten liegen:
 *   MISO
 *   MOSI
 *   SCK
 * übrige Pins auf den äußeren Leisten:
 *   SDA/SCL  für Trackball
 *   8 links
 *   7 rechts, wobei C7 mit einer LED verbunden ist und B5 über 100k an Akku und Masse
 * Das reicht also gerade für 6x7 Tasten, Trackball, die interne LED und Spannungsmessung
 * des Akkus. Wenn der leer ist kann man die übliche Funktion einstellen und alles schlafen 
 * legen, vorher als Warnung die LED blinken lassen. 
 * Gibt es einen Audio-Pin? C6, B5, B6, B7. Mit Akkuversorgung nimmt man also B5 zu dessen
 * Spannungsmessung, ansonsten als Audioausgabe. Die siebte Zeile schließt man an B6 oder 
 * B7, damit mit 36 Tasten und Akku auch eine Audioausgabe möglich ist.
 * 
 * tätsächlich für Akkuspannung freigehalten: B5 Akku oder Audio
 * tatächlich für Audio benutzt: B6 siebte Zeile oder Audio
 * so nicht gemacht, B7 wurde in der 6x6-Matrix benutzt: B7 siebte Zeile oder Audio
 * 
 * Im Fall von Akkuversorgung sollte ein Schalter vorgesehen werden um EN auf Masse zu 
 * ziehen? Nein, einfach ein Schalter in der Leitung zum Akku.
 */

/* key matrix size */
#define MATRIX_ROWS 6
#define MATRIX_COLS 6

/*
 * Keyboard Matrix Assignments
 *
 * Change this to how you wired your keyboard
 * COLS: AVR pins used for columns, left to right
 * ROWS: AVR pins used for rows, top to bottom
 * DIODE_DIRECTION: COL2ROW = COL = Anode (+), ROW = Cathode (-, marked on diode)
 *                  ROW2COL = ROW = Anode (+), COL = Cathode (-, marked on diode)
 *
*/

/* key matrix pins */
// Zeilen links a0 a2 d0 
// Zeilen rechts d12 d11 d6 
#define MATRIX_ROW_PINS { F7, F5, D2, D6, B7, D7 }
// Spalten auf der linken Seite von außen nach innen, zuletzt die Daumen
// d5 a1 a5 a4 a3 d1
#define MATRIX_COL_PINS { C6, F6, F0, F1, F4, D3 }
#define UNUSED_PINS

/* COL2ROW, ROW2COL*/
#define DIODE_DIRECTION ROW2COL

// die Bluetooth-Geschichte ist unsicher, ich will nicht, dass sich jemand verbindet und 
// sofort als Ziel verwendet wird. Dieses undef allein reicht aber nicht aus, wenn man die
// Tastatur ohne USB-Verbindung einschaltet, kann man sich trotzdem einfach per Bluetooth
// verbinden.
#undef OUTPUT_AUTO_ENABLE


#define AUDIO_PIN B6

// Speicher ist knapp, deswegen musste das mal abgeschaltet werden:
//#define NO_MUSIC_MODE
// ...aber inzwischen geht es. Sicher kann man noch Platz sparen, indem die Aufnahme- 
// und Wiedergabemöglichkeiten im qmk-Code auskommentiert werden.

// Deswegen will ich Audio überhaupt haben, es hilft mit mod/tap, dazu muss allerdings
// auch der qmk-Code veränder werden:
#define AUDIO_CLICKY
// 0.01 gibt immer den gleichen Ton
#define AUDIO_CLICKY_FREQ_RANDOMNESS 0.03
#define AUDIO_CLICKY_FREQ_MIN 250
#define AUDIO_CLICKY_FREQ_MAX 800

// Speicher sparen:
// https://github.com/qmk/qmk_firmware/issues/3224#issuecomment-399769416
#define NO_ACTION_MACRO
#define NO_ACTION_FUNCTION

#define DISABLE_LEADER

// https://thomasbaart.nl/2018/12/01/reducing-firmware-size-in-qmk/
#define NO_ACTION_ONESHOT

// https://docs.qmk.fm/#/squeezing_avr
#undef LOCKING_SUPPORT_ENABLE
#undef LOCKING_RESYNC_ENABLE
#define LAYER_STATE_8BIT

