/* Copyright 2021
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "jww3.h"

uint16_t u16CycleTimer;


uint16_t get_tapping_term(uint16_t keycode, keyrecord_t *record) 
{
  // Für die Modifizierer soll es länger sein als für die Layer-Umschalter auf dem Daumen.
  // Mit dem kleinen Finger muss man besonders geduldig sein.
  if (keycode == LSFT_T(KC_E) || 
      keycode == RSFT_T(KC_R))      
    return TAPPING_TERM+20;
  else if (keycode == LALT_T(KC_I) || 
      keycode == LCTL_T(KC_A) || 
      keycode == RCTL_T(KC_N) || 
      keycode == LALT_T(KC_S))      
    return TAPPING_TERM+100;
  else if (keycode == LGUI_T(KC_U) ||       
           keycode == RGUI_T(KC_F))      
    return TAPPING_TERM+150;
  else
    return TAPPING_TERM-50;
}

void sleep_cycletime()
{
  board_prepare_sleep();
  
  ADCSRA = 0; // ADC abgeschaltet
  ACSR = 0x80; // analoger Komparator abgeschaltet  
  
  MCUCR |= 0x80; // jtag disable
  MCUCR |= 0x80; // jtag disable

  // Durch diesen Code die Zykluszeit von void housekeeping_task_user(void) auf größer
  // 8ms zu begrenzen hat die Stromaufnahme der Tastatur mit Adafruit Feather und Pimoroni-Trackball
  // von 24mA auf 21mA reduziert.
  while (timer_elapsed(u16CycleTimer) < 8)
  {
    // laut tmk_core/common/avr/timer.c wird alle 1ms eine ISR ausführt); setPinInput( wenn man hier jetzt
    // schläft, dauert das somit höchstens eine Millisekunde.
    cli();
    //writePinHigh(B3);
    set_sleep_mode(SLEEP_MODE_IDLE);
    sleep_enable();
    sei();
    sleep_cpu();
    sleep_disable();
    //setPinOutput(B3);
    //writePinLow(B3);
  }
  u16CycleTimer = timer_read();
}
