#include QMK_KEYBOARD_H

#ifdef BLUETOOTH_ENABLE
# include "outputselect.h"
#endif

#ifdef MODULE_ADAFRUIT_BLE
# include "adafruit_ble.h"
# include "lufa.h"
#endif

#define U_NP KC_NO // key is not present
#define U_NA KC_NO // present but not available for use
#define U_NU KC_NO // available but not used

enum layers { BASE, NAV, MOUSE, NUM, SYM, FUN };

#define KC_AUML  KC_QUOT
#define KC_OUML  KC_SCLN
#define KC_UUML  KC_LBRC
#define KC_SZETT KC_MINS

const uint16_t PROGMEM keymaps[][MATRIX_ROWS][MATRIX_COLS] = {
  [BASE] = LAYOUT(
    KC_X,         KC_Q,         KC_H,         KC_W,            KC_Y,                                                   KC_C,              KC_T,                 KC_D,         KC_B,         KC_P,
    LGUI_T(KC_U), LALT_T(KC_I), LCTL_T(KC_A), LSFT_T(KC_E),    KC_O,                                                   KC_L,              RSFT_T(KC_R),         RCTL_T(KC_N), LALT_T(KC_S), RGUI_T(KC_F),
    KC_OUML,      KC_AUML,      KC_UUML,      KC_Z,            KC_K,                                                   LT(MOUSE,KC_G),    KC_M,                 KC_J,         KC_V,         KC_SZETT,
                                              LT(FUN, KC_ESC), LT(NAV, KC_SPC),  LT(NUM, KC_TAB),   LT(SYM, KC_ENT),   KC_SPC,            KC_BSPC
                ),
  [NAV] = LAYOUT(
    RESET,   U_NA,    U_NA,    U_NA,    U_NA,                        U_NA,    KC_INS,  KC_DEL,  U_NA,    KC_PGUP,
    KC_LGUI, KC_LALT, KC_LCTL, KC_LSFT, U_NA,                        KC_CAPS, KC_LEFT, KC_UP,   KC_RGHT, KC_PGDN,
    U_NA,    U_NA,    U_NA,    U_NA,    U_NA,                        U_NA,    KC_HOME, KC_DOWN, KC_END,  U_NA,
                               U_NA,    U_NA, U_NA, LT(SYM, KC_ENT), KC_SPC,  KC_BSPC
                ),
  [MOUSE] = LAYOUT(
    U_NA,    AU_TOG,  MU_TOG,  MU_MOD,  CK_TOGG,                   U_NA,    U_NA,     U_NA,    U_NA,    U_NA,
    KC_LGUI, KC_LALT, KC_LCTL, KC_LSFT, CK_UP,                     U_NA,    U_NA,     U_NA,    U_NA,    U_NA,
    U_NA,    U_NA,    OUT_BT,  OUT_USB, CK_DOWN,                   U_NA,    U_NA,     U_NA,    U_NA,    U_NA,
                               U_NA,    U_NA,    U_NA,    KC_BTN1, KC_BTN3, KC_BTN2
                ),
  [NUM] = LAYOUT(
    RALT(KC_7), RALT(KC_0),    LSFT(KC_8), LSFT(KC_9), U_NA,                         U_NA, KC_7, KC_8, KC_9, RESET,
    KC_LGUI,    KC_LALT,       KC_LCTL,    KC_LSFT,    U_NA,                         U_NA, KC_4, KC_5, KC_6, U_NA,
    KC_NUBS,    LSFT(KC_NUBS), RALT(KC_8), RALT(KC_9), U_NA,                         U_NA, KC_1, KC_2, KC_3, U_NA,
                                           U_NA,       U_NA, U_NA, LT(SYM, KC_ENT),  KC_0, KC_BSPC
                ),
  [SYM] = LAYOUT(
    LSFT(KC_EQL), LSFT(KC_0),    KC_NUHS,       LSFT(KC_4), RALT(KC_M),                  LSFT(KC_GRV),  LSFT(KC_COMM),  LSFT(KC_DOT),   RALT(KC_NUBS),  LSFT(KC_NUHS),
    LSFT(KC_6),   RALT(KC_RBRC), RALT(KC_Q),    LSFT(KC_2), RALT(KC_E),                  KC_EQL,        RSFT_T(KC_COMM),RCTL_T(KC_DOT), LSFT(KC_7),     RALT(KC_MINS),
    LSFT(KC_5),   KC_GRV,        LSFT(KC_RBRC), KC_RBRC,    U_NA,                        LSFT(KC_3),    KC_SLSH,        LSFT(KC_SLSH),  LSFT(KC_MINS),  LSFT(KC_1),
                                                U_NA,       U_NA,         U_NA, U_NA,    U_NA,          U_NA
  ),
  [FUN] = LAYOUT(
    U_NA,    U_NA,    U_NA,    U_NA,       U_NA,               U_NA, KC_F7, KC_F8, KC_F9, KC_F12,
    KC_LGUI, KC_LALT, KC_LCTL, KC_LSFT,    U_NA,               U_NA, KC_F4, KC_F5, KC_F6, KC_F11,
    U_NA,    U_NA,    U_NA,    U_NA,       U_NA,               U_NA, KC_F1, KC_F2, KC_F3, KC_F10,
                                           U_NA,  U_NA, U_NA,  U_NA, U_NA,  U_NA
  )
};
 
bool led_update_user(led_t led_state)
{
  board_led_set(led_state.caps_lock);
  
  return true;
}

void keyboard_post_init_user(void) 
{
  board_led_init();
#ifdef BLUETOOTH_ENABLE
  // Eine Bluetooth-Verbindung darf immer erst nach Benutzerinteraktion zustande kommen:
  set_output(OUTPUT_USB);
#endif  
#ifdef MODULE_ADAFRUIT_BLE
  // kein Pullup auf dem Pin mit der Akkuspannung
  writePin(B5, 0);
  setPinInput(B5);
#endif
}

void housekeeping_task_user(void)
{
  PRR0 |= 0x80; // TWI abschalten
  setPinInputHigh(D0);
  setPinInputHigh(D1);
  
  sleep_cycletime();
}
