#include QMK_KEYBOARD_H
//#include "drivers/sensors/pimoroni_trackball.h"
#include "pimoroni_trackball.h"
#include "pointing_device.h"

#ifdef BLUETOOTH_ENABLE
# include "outputselect.h"
#endif

#ifdef MODULE_ADAFRUIT_BLE
# include "adafruit_ble.h"
# include "lufa.h"
#endif

#define U_NP KC_NO // key is not present
#define U_NA KC_NO // present but not available for use
#define U_NU KC_NO // available but not used


enum layers { BASE, NAV, MOUSE, NUM, SYM, FUN, CHAR};

#define KC_AUML  KC_QUOT
#define KC_OUML  KC_SCLN
#define KC_UUML  KC_LBRC
#define KC_SZETT KC_MINS

enum custom_keycodes {
  BALL_SCR = SAFE_RANGE, // scrolls
  BLE_USB_OFF // turn off USB to save power 
};

// todo: KC_M soll gehalten BALL_SCR ergeben.

const uint16_t PROGMEM keymaps[][MATRIX_ROWS][MATRIX_COLS] = {
  [BASE] = LAYOUT(
    KC_X,         KC_Q,         KC_H,         KC_W,            KC_Y,                                                   KC_C,              KC_T,                 KC_D,         KC_B,         KC_P,
    LGUI_T(KC_U), LALT_T(KC_I), LCTL_T(KC_A), LSFT_T(KC_E),    KC_O,                                                   KC_L,              RSFT_T(KC_R),         RCTL_T(KC_N), LALT_T(KC_S), RGUI_T(KC_F),
    KC_OUML,      KC_AUML,      KC_UUML,      KC_Z,            KC_K,                                                   LT(MOUSE,KC_G),    KC_M,                 KC_J,         KC_V,         KC_SZETT,
                                              LT(FUN, KC_ESC), LT(NAV, KC_SPC),  LT(NUM, KC_TAB),   LT(SYM, KC_ENT),   KC_SPC,            KC_BSPC
                ),
  [NAV] = LAYOUT(
    RESET,   U_NA,    U_NA,    U_NA,    U_NA,                        U_NA,    KC_INS,  KC_DEL,  U_NA,    KC_PGUP,
    KC_LGUI, KC_LALT, KC_LCTL, KC_LSFT, U_NA,                        KC_CAPS, KC_LEFT, KC_UP,   KC_RGHT, KC_PGDN,
    U_NA,    U_NA,    U_NA,    U_NA,    U_NA,                        U_NA,    KC_HOME, KC_DOWN, KC_END,  U_NA,
                               U_NA,    U_NA, U_NA, LT(SYM, KC_ENT), KC_SPC,  KC_BSPC
                ),
  [MOUSE] = LAYOUT(
    DF(CHAR),AU_TOG,  MU_TOG,  MU_MOD,  CK_TOGG,                   BLE_USB_OFF, BLE_USB_OFF, BLE_USB_OFF,    BLE_USB_OFF,    BLE_USB_OFF,
    KC_LGUI, KC_LALT, KC_LCTL, KC_LSFT, CK_UP,                     U_NA,    U_NA,     U_NA,    U_NA,    U_NA,
    U_NA,    U_NA,    OUT_BT,  OUT_USB, CK_DOWN,                   U_NA,    BALL_SCR, U_NA,    U_NA,    U_NA,
                               U_NA,    U_NA,    U_NA,    KC_BTN1, KC_BTN3, KC_BTN2
                ),
  [NUM] = LAYOUT(
    RALT(KC_7), RALT(KC_0),    LSFT(KC_8), LSFT(KC_9), U_NA,                         U_NA, KC_7, KC_8, KC_9, RESET,
    KC_LGUI,    KC_LALT,       KC_LCTL,    KC_LSFT,    U_NA,                         U_NA, KC_4, KC_5, KC_6, U_NA,
    KC_NUBS,    LSFT(KC_NUBS), RALT(KC_8), RALT(KC_9), U_NA,                         U_NA, KC_1, KC_2, KC_3, U_NA,
                                           U_NA,       U_NA, U_NA, LT(SYM, KC_ENT),  KC_0, KC_BSPC
                ),
  [SYM] = LAYOUT(
    LSFT(KC_EQL), LSFT(KC_0),    KC_NUHS,       LSFT(KC_4), RALT(KC_M),                  LSFT(KC_GRV),  LSFT(KC_COMM),  LSFT(KC_DOT),   RALT(KC_NUBS),  LSFT(KC_NUHS),
    LSFT(KC_6),   RALT(KC_RBRC), RALT(KC_Q),    LSFT(KC_2), RALT(KC_E),                  KC_EQL,        RSFT_T(KC_COMM),RCTL_T(KC_DOT), LSFT(KC_7),     RALT(KC_MINS),
    LSFT(KC_5),   KC_GRV,        LSFT(KC_RBRC), KC_RBRC,    U_NA,                        LSFT(KC_3),    KC_SLSH,        LSFT(KC_SLSH),  LSFT(KC_MINS),  LSFT(KC_1),
                                                U_NA,       U_NA,         U_NA, U_NA,    U_NA,          U_NA
  ),
  [FUN] = LAYOUT(
    U_NA,    U_NA,    U_NA,    U_NA,       U_NA,               U_NA, KC_F7, KC_F8, KC_F9, KC_F12,
    KC_LGUI, KC_LALT, KC_LCTL, KC_LSFT,    U_NA,               U_NA, KC_F4, KC_F5, KC_F6, KC_F11,
    U_NA,    U_NA,    U_NA,    U_NA,       U_NA,               U_NA, KC_F1, KC_F2, KC_F3, KC_F10,
                                           U_NA,  U_NA, U_NA,  U_NA, U_NA,  U_NA
  ),
  [CHAR] = LAYOUT(
    KC_X,         KC_Q,         KC_H,         KC_W,     KC_Y,                      KC_C, KC_T,  KC_D,         KC_B,         KC_P,
    KC_U,         KC_I,         KC_A,         KC_E,     KC_O,                      KC_L, KC_R,  KC_N,         KC_S,         KC_F,
    KC_OUML,      KC_AUML,      KC_UUML,      KC_Z,     KC_K,                      KC_G, KC_M,  KC_J,         KC_V,         KC_SZETT,
                                              DF(BASE), KC_SPC,  KC_TAB,  KC_ENT,  KC_SPC,      KC_BSPC
  )  
};

#define STATUS_SHOW_CAPSLOCK   0
#define STATUS_SHOW_MOD_STICKY 0
uint8_t uHighestLayer = BASE;
uint8_t uCapsLock = 0;
uint8_t uScrolling = 0;
uint8_t uMod = 0;
uint8_t uStatusLED = 0;
uint8_t uOutBT = 0;
uint8_t uUSBActive = 1;
uint16_t u16BlinkTimer;
uint8_t u8BlinkCnt = 0;


void update_status_led(void)
{
  uint8_t uColNew = uHighestLayer;
  
  if (uMod && STATUS_SHOW_MOD_STICKY > 0)
    uColNew = 64;
  else if (uCapsLock)
    uColNew = 63;
  else if (uScrolling)
    uColNew = 62;
  
#ifdef MODULE_ADAFRUIT_BLE
  switch (u8BlinkCnt & 0x03)
  {
    case 0:
      if (uOutBT)
        uColNew = 61;
      break;
    case 1:
      if (uOutBT && uUSBActive)
        uColNew = 61;
      break;
    case 2:
      {
        uint32_t u32BatVoltage = adafruit_ble_read_battery_voltage();
        // es wird wohl doch nur 0..1023 geliefert entsprechend 3,3V, zusammen mit der externen Beschaltung also 652 bei 4,2V, 558 bei 3,6V
        if (u32BatVoltage > 20 && u32BatVoltage < 3.8*1024*0.5/3.3)
          uColNew = 63;
      }
      break;
    case 3:
      {
        uint32_t u32BatVoltage = adafruit_ble_read_battery_voltage();
        // es wird wohl doch nur 0..1023 geliefert entsprechend 3,3V, zusammen mit der externen Beschaltung also 652 bei 4,2V, 558 bei 3,6V
        if (u32BatVoltage > 20 && u32BatVoltage < 3.6*1024*0.5/3.3)
          uColNew = 63;        
      }
      break;      
  }
  #endif

  if (uStatusLED != uColNew)
  {
    uStatusLED = uColNew;
    
    switch (uStatusLED) 
    {
      case 64:
        trackball_set_rgbw(0,0,0,255); // weiß
        break;
      case 63:
        trackball_set_rgbw(255,0,0,0); // rot
        break;
      case 62:
        trackball_set_rgbw(255,0,255,0); // lila
        break;
      case 61:
        trackball_set_rgbw(0,0,255,0); // blau
        break;
      case NAV:
        trackball_set_rgbw(255,102,0,0); // orange
        break;
      case MOUSE:
        trackball_set_rgbw(100,100,100,0); // mischmasch     
        break;
      case SYM:
        trackball_set_rgbw(0,255,255,0); // türkis
        break;
      case NUM:
        trackball_set_rgbw(0,255,0,0); // grün
        break;
      case FUN:
        trackball_set_rgbw(255,190,0,0); // gelb (mit g=220 war es noch zu grünlich)
        break;      
      default:
        trackball_set_rgbw(0,0,0,0);
        break;
    }
  }
}

layer_state_t layer_state_set_user(layer_state_t state) 
{
  uHighestLayer = get_highest_layer(state);
  if (uHighestLayer == FUN) // Das wird selten benutzt, darum taugt das zum Abschalten.
    uMod = 0;
  update_status_led();
  return state;
}

bool led_update_user(led_t led_state)
{
  if (STATUS_SHOW_CAPSLOCK)
  {
    // CapsLock auf Trackball anzeigen
    uCapsLock = led_state.caps_lock;  
    update_status_led();
  }
  else
  { 
    board_led_set(led_state.caps_lock);
  }
  return true;
}

// keyboards/draculad/keymaps/pimoroni nutzt bool process_record_user  (uint16_t keycode, keyrecord_t *record)
// layouts/community/ergodox/drashna nutzt   bool process_record_keymap(uint16_t keycode, keyrecord_t *record)
// siehe https://docs.qmk.fm/#/custom_quantum_functions?id=example-process_record_user-implementation
bool process_record_user(uint16_t keycode, keyrecord_t *record)
{
  switch (keycode)
  {
    case BALL_SCR:
      if (record->event.pressed)
      {
        uScrolling = 1;
        update_status_led();
        trackball_set_scrolling(true);
      } 
      else
      {
        uScrolling = 0;
        update_status_led();
        trackball_set_scrolling(false);
      }
      return false;
      break;
#ifdef MODULE_ADAFRUIT_BLE    
    case BLE_USB_OFF:
      {
        uint32_t u32BatVoltage = adafruit_ble_read_battery_voltage();
        if (u32BatVoltage > 20 && uOutBT)
        {
          USB_Disable();
          uUSBActive = 0;
          // USB zum Stromsparen gänzlich abschalten.
          PRR1 |= 0x80;
          USBCON |= FRZCLK;
          USBCON &= ~USBE;
        }
      }
      return false;
      break;
#endif
  }

  // Sobald etwas mit Mod gesendet wurde, schalte ich die weiße LED an. Die wird erst
  // Durch einen bestimmten Layerwechsel umgeschaltet, somit bleibt sichtbar, wenn ich
  // (ungewollt) etwas mit einem Mod erzeugt habe.
  // Shift soll nicht angezeigt werden; ich möchte nur ein Zeichen für (versehentliche) Strg, 
  // Alt, Super haben.
  if ((get_mods() & (MOD_BIT(KC_LCTL) | MOD_BIT(KC_RCTL) | MOD_BIT(KC_LALT) | MOD_BIT(KC_LGUI) | MOD_BIT(KC_RGUI))) != 0)
  {
    uMod = 1;
    update_status_led();
  }
  
  return true;
}

void keyboard_post_init_user(void) 
{
  board_led_init();
#ifdef BLUETOOTH_ENABLE
  // Eine Bluetooth-Verbindung darf immer erst nach Benutzerinteraktion zustande kommen:
  set_output(OUTPUT_USB);
#endif  
#ifdef MODULE_ADAFRUIT_BLE
  // kein Pullup auf dem Pin mit der Akkuspannung
  writePin(B5, 0);
  setPinInput(B5);
#endif
}

#ifdef BLUETOOTH_ENABLE 
void set_output_user(uint8_t output) 
{
  if (output == OUTPUT_BLUETOOTH)
  {
    uOutBT = 1;
  }
  else
  {
    uOutBT = 0;
  }
}
#endif

void housekeeping_task_user(void)
{  
#ifdef MODULE_ADAFRUIT_BLE
  // qmk geht ohnehin verschwenderisch mit dem Strom um, dann kann man hier auch
  // ohne Probleme Durchläufe zählen um eine blinkende Anzeige zu generieren etc
  if (timer_elapsed(u16BlinkTimer) > 250)
  {
    u16BlinkTimer = timer_read(); 
    u8BlinkCnt++;
    update_status_led();
  }
  
  if (uUSBActive == 0)
  {
    USB_Disable();
    // USB zum Stromsparen gänzlich abschalten.
    PRR1 |= 0x80;
    USBCON |= FRZCLK;
    USBCON &= ~USBE;
  }
#endif  
  sleep_cycletime();
}
