/* Copyright 2021
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "config_common.h"


/* key matrix size */
#define MATRIX_ROWS 6
#define MATRIX_COLS 6

/*
 * Keyboard Matrix Assignments
 *
 * Change this to how you wired your keyboard
 * COLS: AVR pins used for columns, left to right
 * ROWS: AVR pins used for rows, top to bottom
 * DIODE_DIRECTION: COL2ROW = COL = Anode (+), ROW = Cathode (-, marked on diode)
 *                  ROW2COL = ROW = Anode (+), COL = Cathode (-, marked on diode)
 *
*/

/* key matrix pins */
// Im ersten Layout laufen unter dem Board vier Signale durch und könnten auf
// beiden Seiten angeschlossen werden: b1/d7 b3/e6 f5 f7/c6 (bei den gegenüberliegenden 
// Paaren nur einer der beiden). Zunächst waren die von der linken Seite angeschlossen,
// dann wurde einer verändert, damit der Audio-Pin c6 frei wird.
#define MATRIX_ROW_PINS { D3, D4, B4, F4, F6, B2 }
#define MATRIX_COL_PINS { B6, D2, E6, D7, F7, B5 }
#define UNUSED_PINS
// Sonstiges zur Hardware: B0 und D5 sind als LEDs verfügbar.
// Der auf dem Arduino benannte RX ist D2, TX ist D3.
// Zum Debuggen sind F5/A2, B1/SCK, B3/MISO frei.
// Wenn Pimoroni nicht angeschlossen ist, sind zusätzlich D0/SCL und D1/SDA frei.
// Gar nicht auf die Anschlussleisten des ProMicro herausgeführt sind:
//   E2 -- hart mit GND verbunden
//   F0
//   F1
//   B7
//   C7
//   D6

/* COL2ROW, ROW2COL*/
#define DIODE_DIRECTION ROW2COL

#define AUDIO_PIN C6

#define AUDIO_CLICKY
// 0.01 gibt immer den gleichen Ton
#define AUDIO_CLICKY_FREQ_RANDOMNESS 0.03
#define AUDIO_CLICKY_FREQ_MIN 250
#define AUDIO_CLICKY_FREQ_MAX 800

#ifdef AUDIO_ENABLE
  #define STARTUP_SONG SONG(ODE_TO_JOY)
#endif

// Speicher sparen:
// https://github.com/qmk/qmk_firmware/issues/3224#issuecomment-399769416
#define NO_ACTION_MACRO
#define NO_ACTION_FUNCTION

#define DISABLE_LEADER

// https://thomasbaart.nl/2018/12/01/reducing-firmware-size-in-qmk/
#define NO_ACTION_ONESHOT

// https://docs.qmk.fm/#/squeezing_avr
#undef LOCKING_SUPPORT_ENABLE
#undef LOCKING_RESYNC_ENABLE
#define LAYER_STATE_8BIT
